// -*-c++-*-

/*!
  \file bhv_savior.h
  \brief aggressive goalie behavior
*/

/*
 *Copyright:

 Copyright (C) Hiroki SHIMORA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifndef BHV_SAVIOR_H
#define BHV_SAVIOR_H

#include <rcsc/player/soccer_action.h>
#include <rcsc/geom/vector_2d.h>

class Bhv_Savior
    : public rcsc::SoccerBehavior {
private:
    double translateX( bool reverse, double x ) const;

    bool translateXIsLessEqualsTo( bool reverse,
                                   double x, double threshold ) const;

private:
    rcsc::Vector2D getBasicPosition( rcsc::PlayerAgent * agent,
                                     bool use_conservative_mode,
                                     bool penalty_kick_mode,
                                     bool reverse_x,
                                     bool * in_region,
                                     bool * dangerous,
                                     bool * aggressive_positioning,
                                     bool * emergent_advance ) const;

    rcsc::Vector2D getBasicPositionFromBall
                     ( rcsc::PlayerAgent * agent,
                       const rcsc::Vector2D & ball_pos,
                       double base_dist,
                       const rcsc::Vector2D & self_pos,
                       double defense_line_x,
                       bool use_conservative_mode,
                       bool no_danger_angle,
                       bool * in_region,
                       bool * dangerous ) const;

    bool doPlayOnMove( rcsc::PlayerAgent * agent,
                       bool use_conservative_mode = true,
                       bool penalty_kick_mode = false,
                       bool reverse_x = false );

    bool doKick( rcsc::PlayerAgent * agent );

public:
    bool execute( rcsc::PlayerAgent * agent );
};

#endif
