// -*-c++-*-

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifndef HELIOS_BODY_KICK_TO_CORNER_H
#define HELIOS_BODY_KICK_TO_CORNER_H

#include <rcsc/player/soccer_action.h>
#include <rcsc/geom/vector_2d.h>


class Body_KickToCorner
    : public rcsc::BodyAction {
private:
    const bool M_to_left;


    //! result ball position
    rcsc::Vector2D M_ball_result_pos;
    //! result ball velocity
    rcsc::Vector2D M_ball_result_vel;
    //! estimated kick step
    int M_kick_step;
public:
    // if left == ( world.self().pos().y < 0.0 )
    //   kick to the near side corner
    explicit
    Body_KickToCorner( const bool left )
        : M_to_left( left )
        , M_ball_result_pos( rcsc::Vector2D::INVALIDATED )
        , M_ball_result_vel( rcsc::Vector2D::INVALIDATED )
        , M_kick_step( 0 )
      { }

    bool execute( rcsc::PlayerAgent * agent );

    /*!
      \brief get the result ball position
      \return ball position after kick
     */
    const
    rcsc::Vector2D & ballResultPos() const
      {
          return M_ball_result_pos;
      }

    /*!
      \brief get the result ball velocity
      \return ball velocity after kick
     */
    const
    rcsc::Vector2D & ballResultVel() const
      {
          return M_ball_result_vel;
      }

    /*!
      \brief get the estimated kick steps
      \return estimated kick steps
     */
    int kickStep() const
      {
          return M_kick_step;
      }
};

#endif
