// -*-c++-*-

/*!
  \file bhv_find_player.cpp
  \brief search specified player by body and neck
*/

/*
 *Copyright:

 Copyright (C) Hiroki SHIMORA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "rcsc-ext/action/bhv_find_player.h"
#include <rcsc/player/player_agent.h>
#include <rcsc/player/abstract_player_object.h>
#include <rcsc/action/body_turn_to_point.h>
#include <rcsc/action/neck_turn_to_player_or_scan.h>

namespace rcsc_ext {

/*-------------------------------------------------------------------*/
/*!

*/
Bhv_FindPlayer::Bhv_FindPlayer( const rcsc::AbstractPlayerObject * target_player,
                                int count_threshold )
    : M_target_player( target_player )
    , M_count_threshold( count_threshold )
{
}

/*-------------------------------------------------------------------*/
/*!

*/
bool
Bhv_FindPlayer::execute( rcsc::PlayerAgent * agent )
{
    if ( M_target_player == static_cast<const rcsc::AbstractPlayerObject *>(0) )
    {
        return false;
    }

    if ( M_target_player->seenPosCount() <= M_count_threshold
      && M_target_player->posCount() <= M_count_threshold )
    {
        return false;
    }

    rcsc::Vector2D point = M_target_player->pos() + M_target_player->vel();

    // XXX: if already checked intertia point, should search around!!

    agent->setNeckAction( new rcsc::Neck_TurnToPlayerOrScan( M_target_player, 0 ) );
    rcsc::Body_TurnToPoint( point ).execute( agent );

    return true;
}

}
