#ifndef LIBRCSC_EXT_PLAYER_TYPE_EXT_H
#define LIBRCSC_EXT_PLAYER_TYPE_EXT_H

#include <rcsc/common/player_type.h>
#include <rcsc/geom/angle_deg.h>
#include <rcsc/common/server_param.h>

namespace rcsc_ext {

static
inline
long
PlayerType_cyclesToTurnBody( const rcsc::PlayerType & type,
                             double diff_angle,
                             double player_speed )
{
    double target_angle = std::fabs( diff_angle );

    double turnable_angle( 0.0 );
    double sp = player_speed;

    for ( long  t = 0; ; ++ t )
    {
        if ( turnable_angle >= target_angle )
        {
            return( t );
        }

        turnable_angle += type.effectiveTurn( rcsc::ServerParam::i().maxMoment(), sp );

        sp *= type.playerDecay();
    }

    /* NOTREACHED */
}

static
inline
double
PlayerType_getStopPower( const rcsc::PlayerType & type, double current_speed )
{
    // XXX: effort
    double power = - current_speed / type.dashPowerRate();

    if ( power < rcsc::ServerParam::i().maxPower() )
    {
        power = rcsc::ServerParam::i().maxPower();
    }

    if ( power > rcsc::ServerParam::i().minPower() )
    {
        power = rcsc::ServerParam::i().minPower();
    }

    return power;
}

static
inline
double
PlayerType_momentForTurn( const rcsc::PlayerType & type,
                         const double dir,
                         double self_speed,
                         bool * succeeded = static_cast<bool *>(0) )
{
    try
    {
        double moment = dir * (1.0 + type.inertiaMoment() * self_speed);

        if ( succeeded )
        {
            *succeeded = true;
        }

        if ( moment > rcsc::ServerParam::i().maxMoment() )
        {
            if ( succeeded )
            {
                *succeeded = false;
            }

            moment = rcsc::ServerParam::i().maxMoment();
        }
        else if ( moment < rcsc::ServerParam::i().minMoment() )
        {
            if ( succeeded )
            {
                *succeeded = false;
            }

            moment = rcsc::ServerParam::i().minMoment();
        }

        return( moment );
    }
    catch(...)
    {
        if ( succeeded )
        {
            *succeeded = false;
        }

        return 0.0;
    }
}


static
inline
double
PlayerType_getDashPowerToReachNextCycle( const rcsc::PlayerType & type,
                                         double dist )
{
    // XXX: effort

    double needed_power = dist / type.dashPowerRate();
    double max_power = rcsc::ServerParam::i().maxPower();
    double min_power = rcsc::ServerParam::i().minPower();

    if ( needed_power >= max_power )
    {
        return max_power;
    }
    else if ( needed_power <= min_power )
    {
        return min_power;
    }
    else
    {
        return needed_power;
    }
}

}

#endif
