// -*-c++-*-

/*!
  \file simulator_util.h
  \brief convenience tools related simulator.
*/

/*
 *Copyright:

 Copyright (C) Hiroki SHIMORA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifndef SIMULATOR_UTIL_H_INCLUDED
#define SIMULATOR_UTIL_H_INCLUDED

#include <rcsc/player/soccer_action.h>
#include <rcsc/player/abstract_player_object.h>
#include <rcsc/geom/vector_2d.h>
#include <boost/shared_ptr.hpp>
#include <vector>

class SimulatorUtil {
private:
    /*!
      \brief private constructor, inhibit instantiation
    */
    SimulatorUtil();

public:
    /*!
      \class KickPlan
      \brief ...
    */
    class KickPlan {
    private:
        boost::shared_ptr< rcsc::BodyAction > M_action;
        rcsc::Vector2D M_kick_result_pos;
        rcsc::Vector2D M_kick_result_vel;
        int M_kick_step;

    public:
        /*!
          \brief copy constructor
          \param kp prototype of copy
        */
        KickPlan( const KickPlan & kp )
            : M_action( kp.getAction() )
            , M_kick_result_pos( kp.getKickResultPos() )
            , M_kick_result_vel( kp.getKickResultVel() )
            , M_kick_step( kp.getKickStep() )
        {
        }

        KickPlan( const boost::shared_ptr< rcsc::BodyAction > & action,
                  const rcsc::Vector2D & kick_result_pos,
                  const rcsc::Vector2D & kick_result_vel,
                  int kick_step )
            : M_action( action )
            , M_kick_result_pos( kick_result_pos )
            , M_kick_result_vel( kick_result_vel )
            , M_kick_step( kick_step )
        {
        }

        /*!
          \brief check validity
          \return true if this is valid.
        */
        bool isValid() const
        {
            return M_action;
        }

        boost::shared_ptr< rcsc::BodyAction > getAction() const
        {
            return M_action;
        }

        const rcsc::Vector2D & getKickResultPos() const
        {
            return M_kick_result_pos;
        }

        const rcsc::Vector2D & getKickResultVel() const
        {
            return M_kick_result_vel;
        }

        int getKickStep() const
        {
            return M_kick_step;
        }
    };

    typedef std::vector< KickPlan > KickPlanCont;

    struct SimulatorEstimatedSituation {
    public:
        KickPlan plan;

        long self_access_step;
        long teammate_access_step;
        long opponent_access_step;
        long unknown_access_step;
        int first_access_teammate;
        int first_access_opponent;

        SimulatorEstimatedSituation( const KickPlan & plan,
                                     long self_access_step,
                                     long teammate_access_step,
                                     long opponent_access_step,
                                     long unknown_access_step,
                                     int first_access_teammate,
                                     int first_access_opponent )
            : plan( plan )
            , self_access_step( self_access_step )
            , teammate_access_step( teammate_access_step )
            , opponent_access_step( opponent_access_step )
            , unknown_access_step( unknown_access_step )
            , first_access_teammate( first_access_teammate )
            , first_access_opponent( first_access_opponent )
        {
        }
    };

public:
    static
    boost::shared_ptr< SimulatorEstimatedSituation >
    getBestChoice( std::pair< bool, double >
                   (*evaluator)( const SimulatorEstimatedSituation & s,
                                 rcsc::PlayerAgent * agent ),
                   SimulatorUtil::KickPlanCont & candidates,
                   const rcsc::AbstractPlayerCont & players,
                   rcsc::PlayerAgent * agent,
                   bool self_add_kick_step,
                   int additional_self_penalty_step,
                   bool teammate_add_kick_step,
                   int additional_teammate_penalty_step,
                   bool opponent_add_kick_step,
                   int additional_opponent_penalty_step );
};

#endif
